// 设备数据
let devices = [];

// 服务器数据
let serverInfo = {
  // 标题
  titles: { Ip: "Server address", Port: "Server port", Username: "Username", Password: "Password", LoginStringFormat: "Login string format", DeviceOfflineTimeSec: "Device offline time (sec)" },
  // 可编辑的项
  editableKeys: ['Port', 'Username', 'Password', 'DeviceOfflineTimeSec'],
  // 数据
  data: {},
};

//菜单定义
let menu = {
  // 菜单项
  items: [
    // name为一级菜单名；display为默认展开项；page为页面标识
    // sbuItems中的name为子菜单项的名称
    { name: "Config", display: false, page: "config" },
    { name: "Device list", display: false, items: devices, showCount: true, page: "devices" },
    { name: "Help", display: false, page: "help" },
  ],
  //当前展开的菜单序号
  activeIndex: 1,
};

/**
 * 检测按钮按下
 * @param {*} e 
 */
document.onkeydown = function (e) {
  //键盘按键控制
  e = e || window.event;
  //F5
  if (e.keyCode == 116) {
    location.reload();
  }
}

//Vue解析
const app = new Vue({
  el: '#app',
  data: {
    serverInfo,                     //服务器信息
    menu,                           //菜单
    currentPage: "config",          //当前显示页面
    devices,                        //设备
    deviceStates: ["offline", "connected", "login failure", "login success","data communication"],  //设备状态
  },
  methods: {
    //菜单点击
    menuClick(item) {
      //记录前次状态（因为后面全部不显示）
      let lastState = item.display;
      //上次记录的菜单隐藏
      this.menu.items[this.menu.activeIndex].display = false;
      //切换点击栏的显示状态
      item.display = !lastState;
      //记录这次菜单
      this.menu.activeIndex = this.menu.items.indexOf(item);
      this.currentPage = item.page;
    },
    /**
     * 返回菜单名称及子菜单数量,例如:name (3)
     * @param {*} item 一级菜单项
     * @returns 
     */
    getItemName(item) {
      // 如果有items是数组
      if (isArray(item.items)) {
        return item.name + ("showCount" in item ? "({0})".format([item.items.length]) : "");
      }
      return item.name;
    },
    // 根据序号删除设备
    delDevice(index){
      this.devices.splice(index,1);
    },
    //判断是否允许编辑
    serverInfoEditable(key){
      return this.serverInfo.editableKeys.includes(key)
    },
    // 修改服务器信息
    editServerInfo(e){
      // 不能使用e.currentTarget.name获得name,可以获得id
      let id = e.currentTarget.id;
      this.$set(this.serverInfo.data,id,e.currentTarget.innerText);
      sysInvoke("SetServerInfo",JSON.stringify(this.serverInfo.data));
    },
  },
  filters: {
    /**
     * @param {*} state 过滤器前面的值，会自动传入
     * @param {*} states 过滤器使用时要传递的参数；设备状态描述；例如deviceStates
     * @returns 
     */
    showState(state, states) {
      return states[state];
    },
  }
});

/**
 * 服务器状态更新的方法
 * @param {*} data json字符串 
 */
function serverInfoUpdate(data) {
  // alert(data);
  data = JSON.parse(data);
  for (key in data) {
    app.$set(app.serverInfo.data, key, data[key]);
  }
  // 有State这个属性时
  if (typeof app.serverInfo.data.State != "undefined") {
    // 修改标题
    document.title = "CWT DTU - SCADA tool (Service {0})".format([app.serverInfo.data.State ? "started" : "not start"]);
  }
}

/**
 * 设备状态更新的方法
 * @param {*} data json字符串；数组或对象；包含id属性
 */
function deviceUpdate(data) {
  // alert(data);
  try {
    // 将字符串转为json
    data = JSON.parse(data);
    // 如果是多个设备状态(要求清空原有数据)
    if (isArray(data)) {
      //清空所有设备
      devices.splice(0, devices.length);
      // 添加数据中的设备 
      for (i in data) {
        devices.push(data[i]);
      }
    }
    //不是数组
    else {
      // 是否找到同样id的数据的标识
      let find = false;
      // 循环查找是不是已有设备
      for (i in devices) {
        // id或Ip匹配，是同一个设备
        if (devices[i].Id == data.Id || devices[i].Ip == data.Ip) {
          // 未找到
          if (!find) {
            //前面没有找到则替换 
            devices.splice(i, 1, find ? null : data);
          } else {
            //前面已经找到1个(是有可能有多个的)，则删除
            devices.splice(i, 1);
          }
          // 标识已经找到，方便后面处理
          find = true;
          //找到也不退出，因为可能后面还有id不同但是ip相同的数据
        }
      }
      // 不是已知的设备
      if (!find) {
        // 放到最后
        devices.push(data);
      }
    }
  } catch (err) {
    alert(err);
  }
}

/**
 * 测试更新服务器状态
 */
function test_serverInfoUpdate() {
  let testdata = {
    Ip: "0.0.0.0",
    Port: 3000,
    State: 0,
    Username: "Test",
    Password: "123456",
    LoginStringFormat: "",
    DeviceOfflineTimeSec: 600
  };
  testdata.LoginStringFormat = testdata.Username + "&" + testdata.Password + "&[ID]";
  serverInfoUpdate(JSON.stringify(testdata));
}

/**
 * 测试更新设备数据
 */
function test_deviceUpdate(count) {
  let testdata = [
    { "Name": "Temp1", "Id": "10000001", "Ip": "192.168.1.1", "State": 0, "LastDataTime": "2021-06-04 20:00" },
    { "Name": "Temp2", "Id": "10000002", "Ip": "192.168.1.2", "State": 1, "LastDataTime": "2021-06-05 23:07" },
    { "Name": "Temp3", "Id": "10000003", "Ip": "192.168.1.3", "State": 2, "LastDataTime": "2021-06-06 11:08" },
  ];
  // testdata = { "Name": "温度3", "Id": "10000001", "Ip": "192.168.1.5", "State": 0, "LastDataTime": "2021-07-12 16:17" };
  deviceUpdate(JSON.stringify(testdata));
}

window.onload = function () {
  // 选中默认的菜单
  app.menuClick(app.menu.items[app.menu.activeIndex]);
  // 显示系统提示的方法
  // sysInvoke("Help","showSystemMethods");
  // 设置服务器状态更新的方法
  sysInvoke("SetServerInfoUpdateFun", "serverInfoUpdate");
  // 设置设备状态变更的处理方法
  sysInvoke("SetDeviceInfoUpdateFun", "deviceUpdate");
  //获得当前连接的设备
  sysInvoke("GetDevicesInfo","");
  // 网页直接打开时
  if (typeof Copyright == "undefined") {
    test_serverInfoUpdate();
    test_deviceUpdate();
  }
}

